/* 
 * Dinu Bianca si Manea Stefan
 * PM 2012
 * Line follower
 */

#ifndef F_CPU
#define F_CPU 16000000UL
#endif

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define DELAY 	50
#define DOWN	0
#define UP	1

/* functie care comanda L298N */
void go(unsigned char speedLeft, unsigned char speedRight)
{
	/* dreapta */
	if(speedRight == UP)
		PORTC |= (1 << PC0);
	else
		PORTC &= ~(1 << PC0);
	
	/* stanga */
	if(speedLeft == UP)
		PORTC |= (1 << PC3);
	else
		PORTC &= ~(1<< PC3);
}

/*
 * motorul se opreste cand dau aceleasi valori de comanda 0-0 sau 1-1
 * eu ii dau 0-0
 */
void clear()
{
	PORTC &=~ ((1 << PC0) | (1 << PC2));
	PORTC &=~ ((1 << PC3) | (1 << PC4));
}

/* initializari */
void init()
{
	/* port C */
	DDRC = 0xFF;
	PORTC = 0x00;
	
	/* port A */
	DDRA = 0x00;
	PORTA = 0x07;
	
	/* port D */
	DDRD = 0xFF;
	PORTD = 0x00;

	/* Enable motoare */
	PORTD |= (1<<PD5);
	PORTD |= (1<<PD4);
}

int main()
{
	/*
	 * 0 - centru
	 * 1 - ultima curba a fost stanga
	 * 2 - ultima curba a fost dreapta
	 */
	int last = 0; //asezam masina cu senzorul din mijloc pe linia neagra
	
	/* init */
	init();
	
	while(1)
	{
		/* ledul sta stins cat timp masina stie ce decizie sa ia */
		PORTD &= ~(1<<PD7);

		/* Avem 8 combinatii de inputuri de la 3 senzori*/
		if(!(PINA & (1 << PINA0))){
			if(!(PINA & (1 << PINA1)))
				if(!(PINA & (1 << PINA2))){ 	//0-0-0 toti senzorii dau 0
					if(last == 0){			//sunt pierdut
						clear();			//ma opresc
						PORTD |= (1<<PD7);	//anunt ca nu mai stiu ce trebuie sa fac
					}
					else if(last == 1)		//ultimul senzor a fost stanga deci merg la dreapta
						go(DOWN, UP);
					else
						go(UP, DOWN);		//ma duc la stanga
				}
				else {				//0-0-1 stanga pur
					last = 1;
					go(DOWN, UP);			//dreapta mult
				}
			else{
				if (!(PINA & (1 << PINA2))){ 	//0-1-0 centru
					last = 0;
					go(UP, UP);			//inainte
				}
				else {				//0-1-1 centru-stanga
					last = 1;
					go(DOWN, UP);			//dreapta
				}
			}
		}
		else{
			if(!(PINA & (1 << PINA1))){
				if(!(PINA & (1 << PINA2))){	//1-0-0 mult dreapta
					last = 2;
					go(UP, DOWN);			//ma duc la stanga
				}
			}
			else{
				if (!(PINA & (1 << PINA2))){	//1-1-0 dreapta-centru
					last = 2;
					go(UP, DOWN);			//viraj la stanga
				}
				else {				//1-1-1 intersectie
					go(UP, UP);
				}
			}
		}

		/* wait */
		_delay_ms(DELAY);
	}

	return 0;
}
